use std::sync::Arc;

use crate::commands::tally_by_source::TallyBySource;

use super::command_base::DeserializedCommand;

pub fn deserialize_commands(payload: &[u8]) -> Vec<Arc<dyn DeserializedCommand>> {
    let mut parsed_commands = vec![];
    let mut head = 0;

    while payload.len() > head + 8 {
        // log::debug!("Head at {} out of {}", head, payload.len());
        let length = u16::from_be_bytes([payload[head], payload[head + 1]]) as usize;
        let Ok(name) = String::from_utf8(payload[(head + 4)..(head + 8)].to_vec()) else {
            break;
        };

        if length < 8 {
            break;
        }

        log::debug!("Received command {} with length {}", name, length);

        match name.as_str() {
            "TlSr" => {
                let tally = TallyBySource::deserialize(&payload[head + 8..head + length]);
                for (source, state) in tally.sources {
                    log::info!(
                        "Source {} Program: {}, Preview: {}",
                        source,
                        state.program,
                        state.preview
                    )
                }
            }
            _ => {}
        }

        head += length;
    }

    parsed_commands
}
